--[[
AutoProfitY - Automatically sells low quality gray items and selected white items to the vendor.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:
	
Free Software Foundation, I.,
51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
--]]

local Addon  = LibStub( 'AceAddon-3.0'):GetAddon( 'AutoProfitY')
local L      = LibStub( 'AceLocale-3.0'):GetLocale( 'AutoProfitY')
local Module = Addon:NewModule( 'Options', 'AceEvent-3.0')

local LINK_MATCH = '|c%x+|Hitem:[%-%d:]*|h%[.-%]|h|r'
local SPIN_DESC = {
	[1] = L.SpinNone,
	[2] = L.SpinOverProfit,
	[3] = L.SpinOver,
	[4] = L.SpinProfit,
}
local SPIN_TYPES = {
	['none']       = 1,
	['overProfit'] = 2,
	['over']       = 3,
	['profit']     = 4,
}

--[[---------------------------------------------------------------------------------
  Local Menu
------------------------------------------------------------------------------------]]
local function SetEnable( info, value) 
	if value ~= Addon:IsEnabled() then
		if value then
			Addon:Enable()
			Module:SendMessage( 'AUTO_PROFIT_UPDATE')
		else
			Addon:Disable()
		end
	end
end

local function SetDebug( info, value) 
	Addon:ToggleDebugLog( debug)
end

--[[---------------------------------------------------------------------------------
  Main Menu
------------------------------------------------------------------------------------]]
local function GetProperty( info)
	local key = info[#info]
	return Module.profile[key]
end

local function SetProperty( info, value)
	local key = info[#info]
	Module.profile[key] = value
	Module:SendMessage( 'AUTO_PROFIT_UPDATE')
end

local function GetSpin( info)
	return SPIN_TYPES[Module.profile.spin]
end

local function SetSpin( info, value)
	for k,v in pairs(SPIN_TYPES) do
		if v == value then
			Module.profile.spin = k
			Module:SendMessage( 'AUTO_PROFIT_UPDATE')
			break
		end
	end
end

local function SetException( info, value)
	local key = info[#info]
	local exceptions = (key == 'inputLokal') and Module.profile.exceptions or Module.realm.exceptions
	for link in string.gmatch( value, LINK_MATCH) do	
		local itemID = link:match( 'item:(%d+)')
		if itemID then
			if exceptions[itemID] then
				exceptions[itemID] = nil
				Addon:Print( L.ExceptionRem:format( link))
			else
				exceptions[itemID] = true
				Addon:Print( L.ExceptionAdd:format( link))
			end
		else
			Addon:Print( L.Invalid)
		end
	end
	Module:SendMessage( 'AUTO_PROFIT_UPDATE')
end

local function ListExceptions( info)
	local key = info[#info]
	local exceptions = (key == 'listLocal') and Module.profile.exceptions or Module.realm.exceptions
	local empty = true
	for id in pairs( exceptions) do
		local _, link = GetItemInfo( id)
		if link then
			if empty then
				Addon:Print( L.Exceptions)
				empty = false
			end
			Addon:Print( link)
		end
	end
	if empty then
		Addon:Print( L.ExceptionsEmpty)
	end
end

local function PurgeExceptions( info)
	local key = info[#info]
	local exceptions = (key == 'purgeLocal') and Module.profile.exceptions or Module.realm.exceptions
	for k in pairs( exceptions) do
		exceptions[k] = nil
	end
	Addon:Print( L.ExceptionsDeleted)
	Module:SendMessage( 'AUTO_PROFIT_UPDATE')
end

local function ImportExceptionList( info, value)
	local iName, iRealm = strsplit( '@', value)
	if iName then
		if Module.db.account[iRealm] and Module.db.account[iRealm][iName] and Module.db.account[iRealm][iName].exceptions then
			eList = tdeepcopy( Module.db.account[iRealm][iName].exceptions)
			Module.db.account[realm][name].exceptions = eList
			Addon:Print( L.ExceptionsList1:format( iName, iRealm))
		else
			Addon:Print( L.ExceptionsList2:format( iName, iRealm))
		end
	end
end

local function ValidateExceptionList( info, value)
	local tbl = {}
	for rlm, charList in pairs( Module.db.account) do
		for char in pairs( charList) do
			if char ~= name or rlm ~= realm then
				tinsert( tbl, char..'@'..rlm)
			end
		end
	end
	return tbl
end

local main = {
	type = 'group', order = 10, name = 'Main', get = GetProperty, set = SetProperty, handler = Addon, 
	args = {
		release     = { type = 'description', order = 20,  name = Addon.version,  fontSize = 'large' },
		description = { type = 'description', order = 30,  name = L.Description,  fontSize = 'large' },
		space1      = { type = 'description', order = 40,  name = ' ' },
		enabled     = { type = 'toggle',      order = 140, name = L.EnableName,    desc = L.EnableDesc, get = 'IsEnabled',         set = SetEnable },
		debug       = { type = 'toggle',      order = 150, name = L.DebugName,     desc = L.DebugDesc,  get = 'IsDebugLogEnabled', set = SetDebug },
--		space2      = { type = 'description', order = 200, name = ' ' },
		autosell    = { type = 'toggle',      order = 210, name = L.AutoName,      desc = L.AutoDesc },
		report      = { type = 'toggle',      order = 220, name = L.SilentName,    desc = L.SilentDesc },
		profit      = { type = 'toggle',      order = 230, name = L.ProfitName,    desc = L.ProfitDesc },
		soulbound   = { type = 'toggle',      order = 240, name = L.SoulboundName, desc = L.SoulboundDesc },
		limit       = { type = 'toggle',      order = 250, name = L.LimitName,     desc = L.LimitDesc, width = 'full' },
		spin        = { type = 'select',      order = 260, name = L.SpinName,      desc = L.SpinDesc, values = SPIN_DESC, get = GetSpin, set = SetSpin },
--		space3      = { type = 'description', order = 400, name = ' ' },
		inputGlobal = { type = 'input',       order = 410, name = L.GlobalName,      desc = L.GlobalDesc,      set = SetException, usage = L.Usage, width = 'full' },
		listGlobal  = { type = 'execute',     order = 420, name = L.GlobalListName,  desc = L.GlobalListDesc,  func = ListExceptions },
		purgeGlobal = { type = 'execute',     order = 430, name = L.GlobalPurgeName, desc = L.GlobalPurgeDesc, func = PurgeExceptions },
		inputLocal  = { type = 'input',       order = 440, name = L.LokalName,       desc = L.LokalDesc,       set = SetException, usage = L.Usage, width = 'full' },
		listLocal   = { type = 'execute',     order = 450, name = L.LokalListName,   desc = L.LokalListDesc,   func = ListExceptions },
		purgeLocal  = { type = 'execute',     order = 460, name = L.LokalPurgeName,  desc = L.LokalPurgeDesc,  func = PurgeExceptions },
--		space4      = { type = 'description', order = 500, name = ' ' },
--		import      = { type = 'input',       order = 510, name = L.ImportName,    desc = L.ImportDesc, set = ImportExceptionList, validate = ValidateExceptionList },
	}
}

--[[---------------------------------------------------------------------------------
  Class Setup
------------------------------------------------------------------------------------]]
function Module:OnInitialize()
	self:AUTO_PROFIT_UPDATE()
	LibStub( 'AceConfig-3.0'):RegisterOptionsTable( Addon:GetName(), main)
	LibStub( 'AceConfigDialog-3.0'):AddToBlizOptions( Addon:GetName(), Addon:GetName())
end

function Module:OnEnable()
	self:RegisterMessage( 'AUTO_PROFIT_UPDATE')
end

function Module:OnDisable()
	self:UnregisterMessage( 'AUTO_PROFIT_UPDATE')
end

function Module:AUTO_PROFIT_UPDATE()
	self.profile = Addon.db.profile
	self.realm   = Addon.db.realm
end

function Module:GetOptionTable()
	return main
end
