--[[-----------------------------------------------------------------------------
ProfitButton Widget
-------------------------------------------------------------------------------]]
local Type, Version = 'ProfitButton', 1
local AceGUI = LibStub and LibStub( 'AceGUI-3.0', true)
if not AceGUI or (AceGUI:GetWidgetVersion( Type) or 0) >= Version then return end

-- Lua APIs
local select, pairs, print = select, pairs, print

-- WoW APIs
local CreateFrame, UIParent = CreateFrame, UIParent

--[[-----------------------------------------------------------------------------
Scripts
-------------------------------------------------------------------------------]]
local function Control_OnEnter( frame)
	frame.obj:Fire( 'OnEnter')
end

local function Control_OnLeave(frame)
	frame.obj:Fire( 'OnLeave')
end

local function Control_OnClick( frame, ...)
	frame.obj:Fire( 'OnClick', ...)
	AceGUI:ClearFocus()
end

local function Control_OnUpdate( frame, ...)
	frame.obj:Fire( 'OnUpdate', ...)
end

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
	['OnAcquire'] = function( self)
		self:SetHeight( 33)
		self:SetWidth( 33)
		self:SetImage()
		self:SetRotation()
		self:SetDisabled( false)
	end,

	-- ['OnRelease'] = nil,

	['SetImage'] = function( self, path, ...)
		local image = self.image
		image:SetTexture( path)
		if image:GetTexture() then
			local n = select( '#', ...)
			if n == 4 or n == 8 then
				image:SetTexCoord( ...)
			else
				image:SetTexCoord( 0, 1, 0, 1)
			end
		end
	end,

	['SetRotation'] = function( self, rotation)
		if rotation then
			local S = 0.8 -- Sättigung
			local B = 0.8 -- Helligkeit
			local i, H = math.modf( 5 * rotation) -- Farbschattierung
			local p = B * (1 - S)
			local q = B * (1 - S * H)
			local t = B * (1 - S * (1 - H))
			local hi = i % 6
			if hi == 0 then
				self.spot:SetTexture( B, t, p, 1)
			elseif hi == 1 then
				self.spot:SetTexture( q, B, p, 1)
			elseif hi == 2 then
				self.spot:SetTexture( p, B, t, 1)
			elseif hi == 3 then
				self.spot:SetTexture( p, q, B, 1)
			elseif hi == 4 then
				self.spot:SetTexture( t, p, B, 1)
			elseif hi == 5 then
				self.spot:SetTexture( B, p, q, 1)
			end
		else
			self.spot:SetTexture( 0, 0, 0, 0)
		end
	end,

	['SetDisabled'] = function( self, disabled)
		if disabled then
			self.frame:Disable()
			self.image:SetVertexColor( 0.5, 0.5, 0.5, 0.5)
		else
			self.frame:Enable()
			self.image:SetVertexColor( 1, 1, 1)
		end
	end
}

--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local function Constructor()
	local id  = AceGUI:GetNextWidgetNum( Type)

	local frame = CreateFrame( 'Button', 'AceGUI30PlayerButton' .. id, UIParent, 'UIPanelButtonTemplate2')
	frame:EnableMouse(true)
	frame:SetScript( 'OnEnter',  Control_OnEnter)
	frame:SetScript( 'OnLeave',  Control_OnLeave)
	frame:SetScript( 'OnClick',  Control_OnClick)
	frame:SetScript( 'OnUpdate', Control_OnUpdate)
	frame:Hide()

	local image = frame:CreateTexture( nil, 'ARTWORK')
	image:SetPoint( 'TOPLEFT', frame, 'TOPLEFT', 0, 0)
	image:SetPoint( 'BOTTOMRIGHT', frame, 'BOTTOMRIGHT', 0, 0)

	local spot = frame:CreateTexture( nil, 'OVERLAY')
	spot:SetPoint( 'TOPLEFT', frame, 'TOPLEFT', 5, -5)
	spot:SetPoint( 'BOTTOMRIGHT', frame, 'BOTTOMRIGHT', -5, 5)
	spot:SetTexture( 0, 0, 0, 0)
	spot:SetBlendMode( 'ADD')

	local widget = {
		frame = frame,
		image = image,
		spot  = spot,
		type  = Type
	}
	for method, func in pairs(methods) do
		widget[method] = func
	end
	frame.obj = widget

	return AceGUI:RegisterAsWidget(widget)
end

AceGUI:RegisterWidgetType(Type, Constructor, Version)
