--[[-----------------------------------------------------------------------------
ProfitModel Widget
-------------------------------------------------------------------------------]]
local Type, Version = 'ProfitModel', 1
local AceGUI = LibStub and LibStub( 'AceGUI-3.0', true)
if not AceGUI or (AceGUI:GetWidgetVersion( Type) or 0) >= Version then return end

-- Lua APIs
local pairs = pairs

-- WoW APIs
local _G = _G
local CreateFrame, UIParent = CreateFrame, UIParent

--[[-----------------------------------------------------------------------------
Scripts
-------------------------------------------------------------------------------]]
local function Control_OnClick( frame, ...)
	PlaySound( 'igMainMenuOption')
	frame.obj:Fire( 'OnClick', ...)
	AceGUI:ClearFocus()
end

local function Control_OnEnter( frame)
	frame.obj:Fire( 'OnEnter')
end

local function Control_OnLeave( frame)
	frame.obj:Fire( 'OnLeave')
end

local function Control_OnUpdate( frame, ...)
	frame.obj:Fire( 'OnUpdate', ...)
end

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
	['OnAcquire'] = function(self)
		self:SetHeight( 33)
		self:SetWidth( 33)
		self:SetDisabled( false)
	end,

	-- ['OnRelease'] = nil,

	['SetRotation'] = function( self, rotation)
		self.model:SetRotation( rotation)
	end,

	['SetCreature'] = function( self, id)
		self.model:SetCreature( id)
	end,

	['SetUnit'] = function( self, id)
		self.model:SetUnit( id)
	end,

	['SetModel'] = function( self, file)
		self.model:SetModel( file)
	end,

	['SetDisabled'] = function( self, disabled)
		if disabled then
			self.frame:Disable()
		else
			self.frame:Enable()
		end
	end
}

--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local function Constructor()
	local id  = AceGUI:GetNextWidgetNum( Type)

	local frame = CreateFrame( 'Button', 'AceGUI30PlayerModel' .. id, UIParent, 'UIPanelButtonTemplate2')
	frame:EnableMouse( true)
	frame:SetScript( 'OnClick',  Control_OnClick)
	frame:SetScript( 'OnEnter',  Control_OnEnter)
	frame:SetScript( 'OnLeave',  Control_OnLeave)
	frame:SetScript( 'OnUpdate', Control_OnUpdate)
	frame:Hide()
--[[
	local ghosticon = frame:CreateTexture( nil, 'OVERLAY')
	ghosticon:SetBlendMode( 'ADD')
	ghosticon:SetPoint( 'TOPLEFT', frame, 'TOPLEFT', 0, 0)
	ghosticon:SetPoint( 'BOTTOMRIGHT', frame, 'BOTTOMRIGHT', 0, 0)
	ghosticon:SetTexture( 0, 0, 1, 1)
--]]
	local model = CreateFrame( 'PlayerModel', nil, frame)
	model:SetPoint( 'TOPLEFT', frame, 'TOPLEFT', 0, 0)
	model:SetPoint( 'BOTTOMRIGHT', frame, 'BOTTOMRIGHT', 0, 0)

	local widget = {
		num   = id,
		frame = frame,
		model = model,
		type  = Type
	}
	for method,func in pairs( methods) do
		widget[method] = func
	end
	frame.obj = widget

	return AceGUI:RegisterAsWidget( widget)
end

AceGUI:RegisterWidgetType( Type, Constructor, Version)
